@@ -1,30 +1,34 @@
+import { tick } from "svelte";
+import type { Action } from "svelte/action";
+
+import { log_error } from "./log";
+
 /**
  * Add a tooltip showing the keyboard shortcut over the target element.
  * @param target - The target element to show the tooltip on.
  * @returns A function to remove event handler.
  */
-function showTooltip(target: HTMLElement): () => void {
-  const tooltip = document.createElement("div");
-  const isHidden = target.classList.contains("hidden");
-  if (isHidden) {
-    target.classList.remove("hidden");
+function showTooltip(target: HTMLElement, description: string): () => void {
+  const { hidden } = target;
+  if (hidden) {
+    target.hidden = false;
   }
+  const tooltip = document.createElement("div");
   tooltip.className = "keyboard-tooltip";
-  tooltip.innerHTML = target.getAttribute("data-key") || "";
+  tooltip.textContent = description;
   document.body.appendChild(tooltip);
-  const parentCoords = target.getBoundingClientRect();
+  const targetRect = target.getBoundingClientRect();
   // Padded 10px to the left if there is space or centered otherwise
   const left =
-    parentCoords.left +
+    targetRect.left +
     Math.min((target.offsetWidth - tooltip.offsetWidth) / 2, 10);
-  const top =
-    parentCoords.top + (target.offsetHeight - tooltip.offsetHeight) / 2;
+  const top = targetRect.top + (target.offsetHeight - tooltip.offsetHeight) / 2;
   tooltip.style.left = `${left}px`;
-  tooltip.style.top = `${top + window.pageYOffset}px`;
-  return (): void => {
+  tooltip.style.top = `${top + window.scrollY}px`;
+  return () => {
     tooltip.remove();
-    if (isHidden) {
-      target.classList.add("hidden");
+    if (hidden) {
+      target.hidden = true;
     }
   };
 }
@@ -35,12 +39,15 @@ function showTooltip(target: HTMLElement): () => void {
 function showTooltips(): () => void {
   const removes: (() => void)[] = [];
   document.querySelectorAll("[data-key]").forEach((el) => {
-    if (el instanceof HTMLElement) {
-      removes.push(showTooltip(el));
+    const key = el.getAttribute("data-key");
+    if (el instanceof HTMLElement && key !== null) {
+      removes.push(showTooltip(el, key));
     }
   });
-  return (): void => {
-    removes.forEach((r) => r());
+  return () => {
+    removes.forEach((r) => {
+      r();
+    });
   };
 }
 
@@ -60,6 +67,42 @@ function isEditableElement(element: EventTarget | null): boolean {
   );
 }
 
+type UppercaseLetter =
+  | "A"
+  | "B"
+  | "C"
+  | "D"
+  | "E"
+  | "F"
+  | "G"
+  | "H"
+  | "I"
+  | "J"
+  | "L"
+  | "M"
+  | "N"
+  | "O"
+  | "P"
+  | "Q"
+  | "R"
+  | "S"
+  | "T"
+  | "U"
+  | "V"
+  | "W"
+  | "X"
+  | "Y"
+  | "Z";
+type LowercaseLetter = Lowercase<UppercaseLetter>;
+type Letter = UppercaseLetter | LowercaseLetter;
+// This type can be extended as needed to support all the desired
+// key combinations
+type KeyCombo =
+  | "?"
+  | Letter
+  | `${"Control" | "Meta"}+${"d" | "s" | "Enter"}`
+  // d,s,t - journal filters; f - filters; g - reports
+  | `${"d" | "f" | "g" | "s" | "t"} ${Letter}`;
 /** A handler function or an element to click. */
 type KeyboardShortcutAction = ((event: KeyboardEvent) => void) | HTMLElement;
 const keyboardShortcuts = new Map<string, KeyboardShortcutAction>();
@@ -94,6 +137,7 @@ function keydown(event: KeyboardEvent): void {
       event.preventDefault();
       handler.focus();
     } else if (handler instanceof HTMLElement) {
+      event.preventDefault();
       handler.click();
     } else {
       handler(event);
@@ -107,37 +151,48 @@ function keydown(event: KeyboardEvent): void {
 document.addEventListener("keydown", keydown);
 
 /** A type to specify a platform-dependent keyboard shortcut. */
-export type KeySpec = string | { key: string; mac: string };
+export type KeySpec =
+  | KeyCombo
+  | { key: KeyCombo; mac?: KeyCombo; note?: string };
 
-const currentPlatform: "mac" | "key" =
-  typeof navigator !== "undefined" && /Mac/.test(navigator.platform)
-    ? "mac"
-    : "key";
+const isMac =
+  // This still seems to be the least bad way to check whether we are running on macOS or iOS
+  // eslint-disable-next-line deprecation/deprecation
+  navigator.platform.startsWith("Mac") || navigator.platform === "iPhone";
 
-export const modKey = currentPlatform === "mac" ? "Cmd" : "Ctrl";
+export const modKey = isMac ? "Cmd" : "Ctrl";
 
 /**
  * Get the keyboard key specifier string for the current platform.
- * @param keySpec - The key spec.
+ * @param spec - The key spec.
  */
-export function getKeySpecKey(keySpec: KeySpec): string {
-  if (typeof keySpec === "string") {
-    return keySpec;
+function getKeySpecKey(spec: KeySpec): KeyCombo {
+  if (typeof spec === "string") {
+    return spec;
   }
-  return currentPlatform === "mac" ? keySpec.mac : keySpec.key;
+  return isMac ? spec.mac ?? spec.key : spec.key;
+}
+
+/**
+ * Get the keyboard key description.
+ * @param spec - The key spec.
+ */
+function getKeySpecDescription(spec: KeySpec): string {
+  if (typeof spec === "string") {
+    return spec;
+  }
+  const key = isMac ? spec.mac ?? spec.key : spec.key;
+  return spec.note ? `${key} - ${spec.note}` : key;
 }
 
 /**
  * Bind an event handler to a key.
- * @param key - The key to bind.
+ * @param spec - The key to bind.
  * @param handler - The callback to run on key press.
  * @returns A function to unbind the keyboard handler.
  */
-export function bindKey(
-  keySpec: KeySpec,
-  handler: KeyboardShortcutAction
-): () => void {
-  const key = getKeySpecKey(keySpec);
+function bindKey(spec: KeySpec, handler: KeyboardShortcutAction): () => void {
+  const key = getKeySpecKey(spec);
   const sequence = key.split(" ");
   if (sequence.length > 2) {
     // eslint-disable-next-line no-console
@@ -148,7 +203,7 @@ export function bindKey(
     console.warn("Duplicate keyboard shortcut: ", key, handler);
   }
   keyboardShortcuts.set(key, handler);
-  return (): void => {
+  return () => {
     keyboardShortcuts.delete(key);
   };
 }
@@ -160,39 +215,37 @@ export function bindKey(
  * This listener will focus the given node if it is an <input> element and
  * trigger a click on it otherwise.
  */
-export function keyboardShortcut(
-  node: HTMLElement,
-  key?: string
-): { destroy?: () => void } {
-  if (!key) {
-    return {};
-  }
-  node.setAttribute("data-key", key);
-  const destroy = bindKey(key, node);
-
-  return { destroy };
-}
-
-/**
- * Register keyboard shortcuts for all newly added elements with a
- * `data-keyboard-shortcut` attribute.
- */
-export function initCurrentKeyboardShortcuts(): void {
-  // clean up
-  for (const [key, action] of keyboardShortcuts.entries()) {
-    if (action instanceof HTMLElement && !document.contains(action)) {
-      keyboardShortcuts.delete(key);
+export const keyboardShortcut: Action<HTMLElement, KeySpec | undefined> = (
+  node,
+  spec,
+) => {
+  const setup = (s?: KeySpec) => {
+    if (s) {
+      node.setAttribute("data-key", getKeySpecDescription(s));
+      const unbind = bindKey(s, node);
+      return () => {
+        unbind();
+        node.removeAttribute("data-key");
+      };
     }
-  }
-  document.querySelectorAll("[data-keyboard-shortcut]").forEach((element) => {
-    const key = element.getAttribute("data-keyboard-shortcut");
-    if (key && element instanceof HTMLElement) {
-      element.removeAttribute("data-keyboard-shortcut");
-      element.setAttribute("data-key", key);
-      bindKey(key, element);
-    }
-  });
-}
+    return () => {};
+  };
+  let destroy = setup(spec);
+
+  return {
+    destroy,
+    update(new_spec) {
+      destroy();
+      // Await tick so that key bindings that might have been removed from other
+      // elements in the same render are gone.
+      tick()
+        .then(() => {
+          destroy = setup(new_spec);
+        })
+        .catch(log_error);
+    },
+  };
+};
 
 /**
  * Register the keys to show/hide the tooltips
@@ -204,8 +257,10 @@ export function initGlobalKeyboardShortcuts(): void {
       hide();
       document.removeEventListener("mousedown", once);
       document.removeEventListener("keydown", once);
+      document.removeEventListener("scroll", once);
     };
     document.addEventListener("mousedown", once);
     document.addEventListener("keydown", once);
+    document.addEventListener("scroll", once);
   });
 }
