@@ -14,20 +14,22 @@ import { err, ok } from "./result";
  * That is, a function that checks an unknown object to be of a specified type.
  */
 export type Validator<T> = (json: unknown) => Result<T, string>;
+/** A validtor that will never error. */
 export type SafeValidator<T> = (json: unknown) => Ok<T>;
 
+/** Extract the validated type. */
 export type ValidationT<R> = R extends Validator<infer T> ? T : never;
 
 /**
- * Validate as unknown (noop).
+ * Validate with a default value if validation fails.
  */
 export function defaultValue<T>(
   validator: Validator<T>,
-  value: T
+  value: T,
 ): SafeValidator<T> {
   return (json) => {
     const res = validator(json);
-    return res.success ? res : ok(value);
+    return res.is_ok ? res : ok(value);
   };
 }
 
@@ -75,7 +77,7 @@ export const date: Validator<Date> = (json) => {
  * Validate a value to be equal to a constant value.
  */
 export function constant<T extends null | boolean | string | number>(
-  value: T
+  value: T,
 ): Validator<T> {
   return (json) =>
     json === value ? ok(json as T) : err("Expected a constant");
@@ -92,7 +94,7 @@ export function union<T extends unknown[]>(
   return (json) => {
     for (const validator of args) {
       const res = validator(json) as Result<TupleElement<T>, string>;
-      if (res.success) {
+      if (res.is_ok) {
         return res;
       }
     }
@@ -103,8 +105,9 @@ export function union<T extends unknown[]>(
 /**
  * Validator for an object that might be undefined.
  */
-export function optional<T>(validator: Validator<T>): Validator<T | undefined> {
-  return (json) => (json === undefined ? ok(undefined) : validator(json));
+export function optional<T>(validator: Validator<T>): Validator<T | null> {
+  return (json) =>
+    json === undefined || json === null ? ok(null) : validator(json);
 }
 
 /**
@@ -123,7 +126,7 @@ export function array<T>(validator: Validator<T>): Validator<T[]> {
       const result: T[] = [];
       for (const element of json) {
         const res = validator(element);
-        if (res.success) {
+        if (res.is_ok) {
           result.push(res.value);
         } else {
           return res;
@@ -139,18 +142,20 @@ export function array<T>(validator: Validator<T>): Validator<T[]> {
  * Validator for a tuple of fixed length.
  */
 export function tuple<A, B>(
-  decoders: [Validator<A>, Validator<B>]
+  decoders: [Validator<A>, Validator<B>],
 ): Validator<[A, B]> {
   return (json) => {
     if (Array.isArray(json) && json.length === 2) {
       const result = [];
-      for (let i = 0; i < decoders.length; i += 1) {
-        const res = decoders[i](json[i]);
-        if (res.success) {
+      let i = 0;
+      for (const decoder of decoders) {
+        const res = decoder(json[i]);
+        if (res.is_ok) {
           result[i] = res.value;
         } else {
           return res;
         }
+        i += 1;
       }
       return ok(result as [A, B]);
     }
@@ -165,9 +170,6 @@ export function isJsonObject(json: unknown): json is Record<string, unknown> {
   return typeof json === "object" && json !== null && !Array.isArray(json);
 }
 
-// eslint-disable-next-line
-const { hasOwnProperty } = Object.prototype;
-
 /**
  * Validator for an object with some given properties.
  */
@@ -179,13 +181,13 @@ export function object<T>(validators: {
       const obj: Partial<T> = {};
       // eslint-disable-next-line no-restricted-syntax
       for (const key in validators) {
-        if (hasOwnProperty.call(validators, key)) {
+        if (Object.hasOwn(validators, key)) {
           const res = validators[key](json[key]);
-          if (res.success) {
+          if (res.is_ok) {
             obj[key] = res.value;
           } else {
             return err(
-              `Validating object failed at key '${key}': ${res.value}`
+              `Validating object failed at key '${key}': ${res.error}`,
             );
           }
         }
@@ -205,10 +207,10 @@ export function record<T>(decoder: Validator<T>): Validator<Record<string, T>> {
       const ret: Record<string, T> = {};
       for (const [key, value] of Object.entries(json)) {
         const res = decoder(value);
-        if (res.success) {
+        if (res.is_ok) {
           ret[key] = res.value;
         } else {
-          return err(`Validating record failed at key '${key}': ${res.value}`);
+          return err(`Validating record failed at key '${key}': ${res.error}`);
         }
       }
       return ok(ret);
