@@ -1,6 +1,17 @@
-const notificationList = document.createElement("div");
-notificationList.className = "notifications";
-document.body.appendChild(notificationList);
+import { log_error } from "./log";
+
+/** The notification list div, lazily created. */
+const notificationList = (() => {
+  let value: HTMLDivElement | null = null;
+  return () => {
+    if (value === null) {
+      value = document.createElement("div");
+      value.className = "notifications";
+      document.body.appendChild(value);
+    }
+    return value;
+  };
+})();
 
 type NotificationType = "info" | "warning" | "error";
 
@@ -17,12 +28,12 @@ export function notify(
   msg: string,
   // eslint-disable-next-line default-param-last
   cls: NotificationType = "info",
-  callback?: () => void
+  callback?: () => void,
 ): void {
   const notification = document.createElement("li");
   notification.classList.add(cls);
   notification.appendChild(document.createTextNode(msg));
-  notificationList.append(notification);
+  notificationList().append(notification);
   notification.addEventListener("click", () => {
     notification.remove();
     callback?.();
@@ -31,3 +42,13 @@ export function notify(
     notification.remove();
   }, 5000);
 }
+
+/**
+ * Notify the user about an error and log to console.
+ */
+export function notify_err(error: unknown, msg: (e: Error) => string): void {
+  if (error instanceof Error) {
+    notify(msg(error), "error");
+  }
+  log_error(error);
+}
