@@ -1,23 +1,25 @@
 """Parsing and computing budgets."""
-import datetime
+
+from __future__ import annotations
+
 from collections import Counter
 from collections import defaultdict
+from decimal import Decimal
 from typing import Dict
 from typing import List
 from typing import NamedTuple
-from typing import Tuple
 from typing import TYPE_CHECKING
 
-from beancount.core.data import Custom
-from beancount.core.number import Decimal
-
 from fava.core.module_base import FavaModule
 from fava.helpers import BeancountError
 from fava.util.date import days_in_daterange
 from fava.util.date import Interval
 from fava.util.date import number_of_days_in_period
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
+    import datetime
+
+    from fava.beans.abc import Custom
     from fava.core import FavaLedger
 
 
@@ -32,6 +34,7 @@ class Budget(NamedTuple):
 
 
 BudgetDict = Dict[str, List[Budget]]
+"""A map of account names to lists of budget entries."""
 
 
 class BudgetError(BeancountError):
@@ -41,13 +44,13 @@ class BudgetError(BeancountError):
 class BudgetModule(FavaModule):
     """Parses budget entries."""
 
-    def __init__(self, ledger: "FavaLedger") -> None:
+    def __init__(self, ledger: FavaLedger) -> None:
         super().__init__(ledger)
         self.budget_entries: BudgetDict = {}
 
     def load_file(self) -> None:
         self.budget_entries, errors = parse_budgets(
-            self.ledger.all_entries_by_type.Custom
+            self.ledger.all_entries_by_type.Custom,
         )
         self.ledger.errors.extend(errors)
 
@@ -56,10 +59,13 @@ class BudgetModule(FavaModule):
         account: str,
         begin_date: datetime.date,
         end_date: datetime.date,
-    ) -> Dict[str, Decimal]:
+    ) -> dict[str, Decimal]:
         """Calculate the budget for an account in an interval."""
         return calculate_budget(
-            self.budget_entries, account, begin_date, end_date
+            self.budget_entries,
+            account,
+            begin_date,
+            end_date,
         )
 
     def calculate_children(
@@ -67,10 +73,13 @@ class BudgetModule(FavaModule):
         account: str,
         begin_date: datetime.date,
         end_date: datetime.date,
-    ) -> Dict[str, Decimal]:
+    ) -> dict[str, Decimal]:
         """Calculate the budget for an account including its children."""
         return calculate_budget_children(
-            self.budget_entries, account, begin_date, end_date
+            self.budget_entries,
+            account,
+            begin_date,
+            end_date,
         )
 
     def __bool__(self) -> bool:
@@ -78,8 +87,8 @@ class BudgetModule(FavaModule):
 
 
 def parse_budgets(
-    custom_entries: List[Custom],
-) -> Tuple[BudgetDict, List[BudgetError]]:
+    custom_entries: list[Custom],
+) -> tuple[BudgetDict, list[BudgetError]]:
     """Parse budget directives from custom entries.
 
     Args:
@@ -91,7 +100,6 @@ def parse_budgets(
     Example:
         2015-04-09 custom "budget" Expenses:Books "monthly" 20.00 EUR
     """
-
     budgets: BudgetDict = defaultdict(list)
     errors = []
 
@@ -112,7 +120,7 @@ def parse_budgets(
                         entry.meta,
                         "Invalid interval for budget entry",
                         entry,
-                    )
+                    ),
                 )
                 continue
             budget = Budget(
@@ -125,15 +133,16 @@ def parse_budgets(
             budgets[budget.account].append(budget)
         except (IndexError, TypeError):
             errors.append(
-                BudgetError(entry.meta, "Failed to parse budget entry", entry)
+                BudgetError(entry.meta, "Failed to parse budget entry", entry),
             )
 
     return budgets, errors
 
 
 def _matching_budgets(
-    budgets: BudgetDict, accounts: str, date_active: datetime.date
-) -> Dict[str, Budget]:
+    budgets: list[Budget],
+    date_active: datetime.date,
+) -> dict[str, Budget]:
     """Find matching budgets.
 
     Returns:
@@ -141,7 +150,7 @@ def _matching_budgets(
         specified account.
     """
     last_seen_budgets = {}
-    for budget in budgets[accounts]:
+    for budget in budgets:
         if budget.date_start <= date_active:
             last_seen_budgets[budget.currency] = budget
         else:
@@ -154,7 +163,7 @@ def calculate_budget(
     account: str,
     date_from: datetime.date,
     date_to: datetime.date,
-) -> Dict[str, Decimal]:
+) -> dict[str, Decimal]:
     """Calculate budget for an account.
 
     Args:
@@ -167,20 +176,18 @@ def calculate_budget(
         A dictionary of currency to Decimal with the budget for the
         specified account and period.
     """
-    if account not in budgets:
+    budget_list = budgets.get(account, None)
+    if budget_list is None:
         return {}
 
-    currency_dict: Dict[str, Decimal] = defaultdict(Decimal)
+    currency_dict: dict[str, Decimal] = defaultdict(Decimal)
 
-    for single_day in days_in_daterange(date_from, date_to):
-        matches = _matching_budgets(budgets, account, single_day)
+    for day in days_in_daterange(date_from, date_to):
+        matches = _matching_budgets(budget_list, day)
         for budget in matches.values():
-            currency_dict[
-                budget.currency
-            ] += budget.number / number_of_days_in_period(
-                budget.period, single_day
-            )
-    return currency_dict
+            days_in_period = number_of_days_in_period(budget.period, day)
+            currency_dict[budget.currency] += budget.number / days_in_period
+    return dict(currency_dict)
 
 
 def calculate_budget_children(
@@ -188,7 +195,7 @@ def calculate_budget_children(
     account: str,
     date_from: datetime.date,
     date_to: datetime.date,
-) -> Dict[str, Decimal]:
+) -> dict[str, Decimal]:
     """Calculate budget for an account including budgets of its children.
 
     Args:
@@ -201,11 +208,11 @@ def calculate_budget_children(
         A dictionary of currency to Decimal with the budget for the
         specified account and period.
     """
-    currency_dict: Dict[str, Decimal] = Counter()  # type: ignore
+    currency_dict: dict[str, Decimal] = Counter()  # type: ignore[assignment]
 
-    for child in budgets.keys():
+    for child in budgets:
         if child.startswith(account):
             currency_dict.update(
-                calculate_budget(budgets, child, date_from, date_to)
+                calculate_budget(budgets, child, date_from, date_to),
             )
-    return currency_dict
+    return dict(currency_dict)
