@@ -5,31 +5,37 @@ This module contains a list of possible options, the defaults and the code for
 parsing the options.
 
 """
+
 from __future__ import annotations
 
-import datetime
 import re
-from dataclasses import asdict
 from dataclasses import dataclass
 from dataclasses import field
 from dataclasses import fields
-from typing import NamedTuple
 from typing import Pattern
+from typing import TYPE_CHECKING
 
-from babel.core import Locale  # type: ignore
+from babel.core import Locale
 from babel.core import UnknownLocaleError
-from beancount.core.data import Custom
 
+from fava.beans.funcs import get_position
 from fava.helpers import BeancountError
-from fava.util.date import FiscalYearEnd
+from fava.util.date import END_OF_YEAR
 from fava.util.date import parse_fye_string
 
+if TYPE_CHECKING:  # pragma: no cover
+    import datetime
+
+    from fava.beans.abc import Custom
+    from fava.util.date import FiscalYearEnd
+
 
 class OptionError(BeancountError):
     """An error for one the Fava options."""
 
 
-class InsertEntryOption(NamedTuple):
+@dataclass(frozen=True)
+class InsertEntryOption:
     """Insert option.
 
     An option that determines where entries for matching accounts should be
@@ -46,8 +52,6 @@ class InsertEntryOption(NamedTuple):
 class FavaOptions:
     """Options for Fava that can be set in the Beancount file."""
 
-    # pylint: disable=too-many-instance-attributes
-
     account_journal_include_children: bool = True
     auto_reload: bool = False
     collapse_pattern: list[Pattern[str]] = field(default_factory=list)
@@ -55,7 +59,7 @@ class FavaOptions:
     conversion_currencies: tuple[str, ...] = ()
     default_file: str | None = None
     default_page: str = "income_statement/"
-    fiscal_year_end: FiscalYearEnd = FiscalYearEnd(12, 31)
+    fiscal_year_end: FiscalYearEnd = END_OF_YEAR
     import_config: str | None = None
     import_dirs: tuple[str, ...] = ()
     indent: int = 2
@@ -72,53 +76,61 @@ class FavaOptions:
     uptodate_indicator_grey_lookback_days: int = 60
     use_external_editor: bool = False
 
-    asdict = asdict
-
 
 _fields = fields(FavaOptions)
 All_OPTS = {f.name for f in _fields}
-BOOL_OPTS = {f.name for f in _fields if f.type == "bool"}  # type: ignore
-INT_OPTS = {f.name for f in _fields if f.type == "int"}  # type: ignore
+BOOL_OPTS = {f.name for f in _fields if str(f.type) == "bool"}
+INT_OPTS = {f.name for f in _fields if str(f.type) == "int"}
 TUPLE_OPTS = {f.name for f in _fields if f.type.startswith("tuple[str,")}
 STR_OPTS = {f.name for f in _fields if f.type.startswith("str")}
 
 
-def parse_option_custom_entry(entry: Custom, options: FavaOptions) -> None:
+def parse_option_custom_entry(  # noqa: PLR0912
+    entry: Custom,
+    options: FavaOptions,
+) -> None:
     """Parse a single custom fava-option entry and set option accordingly."""
     key = entry.values[0].value.replace("-", "_")
-    assert key in All_OPTS, f"unknown option `{key}`"
+    if key not in All_OPTS:
+        raise ValueError(f"unknown option `{key}`")
 
     if key == "default_file":
-        options.default_file = entry.meta["filename"]
+        filename, _lineno = get_position(entry)
+        options.default_file = filename
         return
 
     value = entry.values[1].value
-    assert isinstance(value, str), f"expected string value for option `{key}`"
+    if not isinstance(value, str):
+        raise TypeError(f"expected string value for option `{key}`")
 
     if key == "insert_entry":
         try:
             pattern = re.compile(value)
-        except re.error:
-            assert False, f"Should be a regular expression: '{value}'."
-        opt = InsertEntryOption(
-            entry.date, pattern, entry.meta["filename"], entry.meta["lineno"]
-        )
+        except re.error as err:
+            raise TypeError(
+                f"Should be a regular expression: '{value}'.",
+            ) from err
+        filename, lineno = get_position(entry)
+        opt = InsertEntryOption(entry.date, pattern, filename, lineno)
         options.insert_entry.append(opt)
     elif key == "collapse_pattern":
         try:
             pattern = re.compile(value)
-        except re.error:
-            assert False, f"Should be a regular expression: '{value}'."
+        except re.error as err:
+            raise TypeError(
+                f"Should be a regular expression: '{value}'.",
+            ) from err
         options.collapse_pattern.append(pattern)
     elif key == "locale":
         try:
             Locale.parse(value)
             options.locale = value
-        except UnknownLocaleError:
-            assert False, f"Unknown locale: '{value}'."
+        except UnknownLocaleError as err:
+            raise ValueError(f"Unknown locale: '{value}'.") from err
     elif key == "fiscal_year_end":
         fye = parse_fye_string(value)
-        assert fye, "Invalid 'fiscal_year_end' option."
+        if fye is None:
+            raise ValueError("Invalid 'fiscal_year_end' option.")
         options.fiscal_year_end = fye
     elif key in STR_OPTS:
         setattr(options, key, value)
@@ -126,8 +138,7 @@ def parse_option_custom_entry(entry: Custom, options: FavaOptions) -> None:
         setattr(options, key, value.lower() == "true")
     elif key in INT_OPTS:
         setattr(options, key, int(value))
-    else:
-        assert key in TUPLE_OPTS, f"unknown option `{key}`"
+    else:  # key in TUPLE_OPTS
         setattr(options, key, tuple(value.strip().split(" ")))
 
 
@@ -147,15 +158,14 @@ def parse_options(
         A tuple (options, errors) where options is a dictionary of all options
         to values, and errors contains possible parsing errors.
     """
-
     options = FavaOptions()
     errors = []
 
     for entry in (e for e in custom_entries if e.type == "fava-option"):
         try:
             parse_option_custom_entry(entry, options)
-        except (IndexError, TypeError, AssertionError) as err:
-            msg = f"Failed to parse fava-option entry: {str(err)}"
+        except (IndexError, TypeError, ValueError) as err:
+            msg = f"Failed to parse fava-option entry: {err!s}"
             errors.append(OptionError(entry.meta, msg, entry))
 
     return options, errors
