@@ -1,20 +1,51 @@
 """Alternative implementation of Beancount's Inventory."""
-from typing import Any
+
+from __future__ import annotations
+
+from decimal import Decimal
 from typing import Callable
 from typing import Dict
+from typing import NamedTuple
 from typing import Optional
 from typing import Tuple
+from typing import TYPE_CHECKING
 
-from beancount.core.amount import Amount
-from beancount.core.number import Decimal
-from beancount.core.number import ZERO
-from beancount.core.position import Cost
-from beancount.core.position import Position
+from fava.beans.protocols import Cost
+
+if TYPE_CHECKING:  # pragma: no cover
+    import datetime
+    from typing import Concatenate
+    from typing import Iterable
+    from typing import Iterator
+    from typing import ParamSpec
+
+    from fava.beans.protocols import Amount
+    from fava.beans.protocols import Position
+
+    P = ParamSpec("P")
 
 
+ZERO = Decimal()
 InventoryKey = Tuple[str, Optional[Cost]]
 
 
+class _Amount(NamedTuple):
+    number: Decimal
+    currency: str
+
+
+class _Cost(NamedTuple):
+    number: Decimal
+    currency: str
+    date: datetime.date
+    label: str | None
+
+
+class _Position(NamedTuple):
+    units: Amount
+    cost: Cost | None
+
+
 class SimpleCounterInventory(Dict[str, Decimal]):
     """A simple inventory mapping just strings to numbers."""
 
@@ -30,6 +61,26 @@ class SimpleCounterInventory(Dict[str, Decimal]):
         else:
             self[key] = new_num
 
+    def __iter__(self) -> Iterator[str]:
+        raise NotImplementedError
+
+    def __neg__(self) -> SimpleCounterInventory:
+        return SimpleCounterInventory({key: -num for key, num in self.items()})
+
+    def reduce(
+        self,
+        reducer: Callable[Concatenate[Position, P], Amount],
+        *args: P.args,
+        **_kwargs: P.kwargs,
+    ) -> SimpleCounterInventory:
+        """Reduce inventory."""
+        counter = SimpleCounterInventory()
+        for currency, number in self.items():
+            pos = _Position(_Amount(number, currency), None)
+            amount = reducer(pos, *args)
+            counter.add(amount.currency, amount.number)
+        return counter
+
 
 class CounterInventory(Dict[InventoryKey, Decimal]):
     """A lightweight inventory.
@@ -53,8 +104,21 @@ class CounterInventory(Dict[InventoryKey, Decimal]):
         else:
             self[key] = new_num
 
+    def __iter__(self) -> Iterator[InventoryKey]:
+        raise NotImplementedError
+
+    @staticmethod
+    def from_positions(positions: Iterable[Position]) -> CounterInventory:
+        inv = CounterInventory()
+        for position in positions:
+            inv.add_position(position)
+        return inv
+
     def reduce(
-        self, reducer: Callable[..., Amount], *args: Any
+        self,
+        reducer: Callable[Concatenate[Position, P], Amount],
+        *args: P.args,
+        **_kwargs: P.kwargs,
     ) -> SimpleCounterInventory:
         """Reduce inventory.
 
@@ -63,15 +127,13 @@ class CounterInventory(Dict[InventoryKey, Decimal]):
         """
         counter = SimpleCounterInventory()
         for (currency, cost), number in self.items():
-            pos = Position(Amount(number, currency), cost)
+            pos = _Position(_Amount(number, currency), cost)
             amount = reducer(pos, *args)
-            assert amount.number is not None
             counter.add(amount.currency, amount.number)
         return counter
 
-    def add_amount(self, amount: Amount, cost: Optional[Cost] = None) -> None:
+    def add_amount(self, amount: Amount, cost: Cost | None = None) -> None:
         """Add an Amount to the inventory."""
-        assert amount.number is not None
         key = (amount.currency, cost)
         self.add(key, amount.number)
 
@@ -79,15 +141,15 @@ class CounterInventory(Dict[InventoryKey, Decimal]):
         """Add a Position or Posting to the inventory."""
         self.add_amount(pos.units, pos.cost)
 
-    def __neg__(self) -> "CounterInventory":
+    def __neg__(self) -> CounterInventory:
         return CounterInventory({key: -num for key, num in self.items()})
 
-    def __add__(self, other: "CounterInventory") -> "CounterInventory":
+    def __add__(self, other: CounterInventory) -> CounterInventory:
         counter = CounterInventory(self)
         counter.add_inventory(other)
         return counter
 
-    def add_inventory(self, counter: "CounterInventory") -> None:
+    def add_inventory(self, counter: CounterInventory) -> None:
         """Add another :class:`CounterInventory`."""
         if not self:
             self.update(counter)
