@@ -1,38 +1,38 @@
 """Some miscellaneous reports."""
+
+from __future__ import annotations
+
 import datetime
 import io
 import re
-from typing import List
-from typing import Tuple
 from typing import TYPE_CHECKING
 
 from beancount.core.amount import CURRENCY_RE
-from beancount.core.data import Custom
-from beancount.core.data import Event
 
 from fava.core.module_base import FavaModule
 from fava.helpers import BeancountError
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.beans.abc import Custom
+    from fava.beans.abc import Event
     from fava.core import FavaLedger
 
+    SidebarLinks = list[tuple[str, str]]
+
 
 class FavaError(BeancountError):
     """Generic Fava-specific error."""
 
 
-SidebarLinks = List[Tuple[str, str]]
-
-
 class FavaMisc(FavaModule):
     """Provides access to some miscellaneous reports."""
 
-    def __init__(self, ledger: "FavaLedger") -> None:
+    def __init__(self, ledger: FavaLedger) -> None:
         super().__init__(ledger)
         #: User-chosen links to show in the sidebar.
         self.sidebar_links: SidebarLinks = []
         #: Upcoming events in the next few days.
-        self.upcoming_events: List[Event] = []
+        self.upcoming_events: list[Event] = []
 
     def load_file(self) -> None:
         custom_entries = self.ledger.all_entries_by_type.Custom
@@ -40,7 +40,7 @@ class FavaMisc(FavaModule):
 
         self.upcoming_events = upcoming_events(
             self.ledger.all_entries_by_type.Event,
-            self.ledger.fava_options["upcoming-events"],
+            self.ledger.fava_options.upcoming_events,
         )
 
         if not self.ledger.options["operating_currency"]:
@@ -50,11 +50,11 @@ class FavaMisc(FavaModule):
                     "No operating currency specified. "
                     "Please add one to your beancount file.",
                     None,
-                )
+                ),
             )
 
 
-def sidebar_links(custom_entries: List[Custom]) -> List[Tuple[str, str]]:
+def sidebar_links(custom_entries: list[Custom]) -> list[tuple[str, str]]:
     """Parse custom entries for links.
 
     They have the following format:
@@ -70,7 +70,7 @@ def sidebar_links(custom_entries: List[Custom]) -> List[Tuple[str, str]]:
     ]
 
 
-def upcoming_events(events: List[Event], max_delta: int) -> List[Event]:
+def upcoming_events(events: list[Event], max_delta: int) -> list[Event]:
     """Parse entries for upcoming events.
 
     Args:
@@ -93,13 +93,12 @@ def upcoming_events(events: List[Event], max_delta: int) -> List[Event]:
 
 
 ALIGN_RE = re.compile(
-    rf'([^";]*?)\s+([-+]?\s*[\d,]+(?:\.\d*)?)\s+({CURRENCY_RE}\b.*)'
+    rf'([^";]*?)\s+([-+]?\s*[\d,]+(?:\.\d*)?)\s+({CURRENCY_RE}\b.*)',
 )
 
 
 def align(string: str, currency_column: int) -> str:
     """Align currencies in one column."""
-
     output = io.StringIO()
     for line in string.splitlines():
         match = ALIGN_RE.match(line)
