@@ -3,34 +3,45 @@
 This mainly serves as an example how Fava's extension systems, which only
 really does hooks at the moment, works.
 """
-# mypy: ignore-errors
-# pylint: disable=missing-docstring
+
 from __future__ import annotations
 
-import os
-import subprocess
+from pathlib import Path
+from subprocess import call
+from subprocess import DEVNULL
+from typing import Any
+from typing import TYPE_CHECKING
 
 from fava.ext import FavaExtensionBase
 
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.beans.abc import Directive
+
 
 class AutoCommit(FavaExtensionBase):
-    def _run(self, args):
-        cwd = os.path.dirname(self.ledger.beancount_file_path)
-        subprocess.call(args, cwd=cwd, stdout=subprocess.DEVNULL)
+    """Auto-commit hook for Fava."""
 
-    def after_write_source(self, path, _):
+    def _run(self, args: list[str]) -> None:
+        cwd = Path(self.ledger.beancount_file_path).parent
+        call(args, cwd=cwd, stdout=DEVNULL)
+
+    def after_write_source(self, path: str, _: Any) -> None:
         message = "autocommit: file saved"
         self._run(["git", "add", path])
         self._run(["git", "commit", "-m", message])
 
-    def after_insert_metadata(self, *_):
+    def after_insert_metadata(self, *_: Any) -> None:
         message = "autocommit: metadata added"
         self._run(["git", "commit", "-am", message])
 
-    def after_insert_entry(self, entry):
+    def after_insert_entry(self, entry: Directive) -> None:
         message = f"autocommit: entry on {entry.date}"
         self._run(["git", "commit", "-am", message])
 
-    def after_entry_modified(self, entry, _):
+    def after_delete_entry(self, entry: Directive) -> None:
+        message = f"autocommit: deleted entry on {entry.date}"
+        self._run(["git", "commit", "-am", message])
+
+    def after_entry_modified(self, entry: Directive, _: Any) -> None:
         message = f"autocommit: modified entry on {entry.date}"
         self._run(["git", "commit", "-am", message])
