@@ -4,24 +4,27 @@ It goes through all entries with a `document` metadata-key, and tries to
 associate them to Document entries. For transactions, it then also adds a link
 from the transaction to documents, as well as the "#linked" tag.
 """
+
+from __future__ import annotations
+
 from collections import defaultdict
-from os.path import basename
-from os.path import dirname
-from os.path import join
 from os.path import normpath
+from pathlib import Path
 from typing import Any
-from typing import List
-from typing import Tuple
+from typing import TYPE_CHECKING
 
-from beancount.core.compare import hash_entry
-from beancount.core.data import Document
-from beancount.core.data import Entries
-from beancount.core.data import Transaction
-
-from fava.core.accounts import get_entry_accounts
+from fava.beans.abc import Document
+from fava.beans.abc import Transaction
+from fava.beans.account import get_entry_accounts
+from fava.beans.funcs import get_position
+from fava.beans.funcs import hash_entry
+from fava.beans.helpers import replace
 from fava.helpers import BeancountError
 from fava.util.sets import add_to_set
 
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.beans.abc import Directive
+
 
 class DocumentError(BeancountError):
     """Document-linking related error."""
@@ -31,10 +34,10 @@ __plugins__ = ["link_documents"]
 
 
 def link_documents(
-    entries: Entries, _: Any
-) -> Tuple[Entries, List[DocumentError]]:
+    entries: list[Directive],
+    _: Any,
+) -> tuple[list[Directive], list[DocumentError]]:
     """Link entries to documents."""
-
     errors = []
 
     # All document indices by their full file path.
@@ -45,13 +48,13 @@ def link_documents(
     for index, entry in enumerate(entries):
         if isinstance(entry, Document):
             by_fullname[entry.filename] = index
-            by_basename[basename(entry.filename)].append((index, entry))
+            by_basename[Path(entry.filename).name].append((index, entry))
 
     for index, entry in enumerate(entries):
         disk_docs = [
             value
             for key, value in entry.meta.items()
-            if key.startswith("document")
+            if key.startswith("document") and isinstance(value, str)
         ]
 
         if not disk_docs:
@@ -59,6 +62,7 @@ def link_documents(
 
         hash_ = hash_entry(entry)[:8]
         entry_accounts = get_entry_accounts(entry)
+        entry_filename, _ = get_position(entry)
         for disk_doc in disk_docs:
             documents = [
                 j
@@ -66,7 +70,7 @@ def link_documents(
                 if document.account in entry_accounts
             ]
             disk_doc_path = normpath(
-                join(dirname(entry.meta["filename"]), disk_doc)
+                Path(entry_filename).parent / disk_doc,
             )
             if disk_doc_path in by_fullname:
                 documents.append(by_fullname[disk_doc_path])
@@ -77,15 +81,16 @@ def link_documents(
                         entry.meta,
                         f"Document not found: '{disk_doc}'",
                         entry,
-                    )
+                    ),
                 )
                 continue
 
             for j in documents:
                 # Since we might link a document multiple times, we have to use
                 # the index for the replacement here.
-                doc: Document = entries[j]  # type: ignore
-                entries[j] = doc._replace(
+                doc: Document = entries[j]  # type: ignore[assignment]
+                entries[j] = replace(
+                    doc,
                     links=add_to_set(doc.links, hash_),
                     tags=add_to_set(doc.tags, "linked"),
                 )
@@ -93,8 +98,9 @@ def link_documents(
             # The other entry types do not support links, so only add links for
             # txns.
             if isinstance(entry, Transaction):
-                entries[index] = entry._replace(
-                    links=add_to_set(entry.links, hash_)
+                entries[index] = replace(
+                    entry,
+                    links=add_to_set(entry.links, hash_),
                 )
 
     return entries, errors
