@@ -1,21 +1,27 @@
 """Tests for Fava's main Flask app."""
+
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+from urllib.parse import urljoin
+
 import pytest
-import werkzeug.routing
-import werkzeug.urls
 from beancount import __version__ as beancount_version
-from flask import Flask
-from flask import url_for
-from flask.testing import FlaskClient
-from pytest import MonkeyPatch
 
-from .conftest import SnapshotFunc
 from fava import __version__ as fava_version
-from fava.application import REPORTS
+from fava.application import create_app
+from fava.application import SERVER_SIDE_REPORTS
 from fava.application import static_url
 from fava.context import g
 
+if TYPE_CHECKING:  # pragma: no cover
+    from pathlib import Path
+
+    from flask import Flask
+    from flask.testing import FlaskClient
+
+    from .conftest import SnapshotFunc
+
 FILTER_COMBINATIONS = [
     {"account": "Assets"},
     {"filter": "any(account: Assets)"},
@@ -24,47 +30,44 @@ FILTER_COMBINATIONS = [
 
 
 @pytest.mark.parametrize(
-    "report,filters",
+    ("report", "filters"),
     [
         (report, filters)
-        for report in REPORTS
+        for report in SERVER_SIDE_REPORTS
         for filters in FILTER_COMBINATIONS
     ],
 )
 def test_reports(
-    app: Flask, test_client: FlaskClient, report: str, filters: dict[str, str]
+    test_client: FlaskClient,
+    report: str,
+    filters: dict[str, str],
 ) -> None:
     """The standard reports work without error (content isn't checked here)."""
-    with app.test_request_context("/long-example/"):
-        app.preprocess_request()
-        url = url_for("report", report_name=report, **filters)
-
-    result = test_client.get(url)
+    result = test_client.get(f"/long-example/{report}/", query_string=filters)
     assert result.status_code == 200
 
 
-@pytest.mark.parametrize("filters", FILTER_COMBINATIONS)
-def test_account_page(
-    app: Flask, test_client: FlaskClient, filters: dict[str, str]
+def test_client_side_reports(
+    test_client: FlaskClient,
+    snapshot: SnapshotFunc,
 ) -> None:
-    """Account page works without error."""
-    for subreport in ["journal", "balances", "changes"]:
-        with app.test_request_context("/long-example/"):
-            app.preprocess_request()
-            url = url_for(
-                "account",
-                name="Assets:US:BofA:Checking",
-                subreport=subreport,
-                **filters,
-            )
-
-        result = test_client.get(url)
-        assert result.status_code == 200
+    """The client-side rendered reports are generated."""
+    result = test_client.get("/long-example/documents/")
+    assert result.status_code == 200
+    snapshot(result.get_data(as_text=True))
 
 
 @pytest.mark.parametrize(
-    "url,return_code",
-    [("/", 302), ("/asdfasdf/", 404), ("/asdfasdf/asdfasdf/", 404)],
+    ("url", "return_code"),
+    [
+        ("/", 302),
+        ("/asdfasdf/", 404),
+        ("/asdfasdf/asdfasdf/", 404),
+        ("/example/not-a-report/", 404),
+        ("/example/holdings/not-a-holdings-aggregation-key/", 404),
+        ("/example/holdings/by_not-a-holdings-aggregation-key/", 404),
+        ("/example/account/Assets:US:BofA:Checking/not_a_subreport/", 404),
+    ],
 )
 def test_urls(test_client: FlaskClient, url: str, return_code: int) -> None:
     """Some URLs return a 404."""
@@ -73,7 +76,7 @@ def test_urls(test_client: FlaskClient, url: str, return_code: int) -> None:
 
 
 @pytest.mark.parametrize(
-    "url,option,expect",
+    ("url", "option", "expect"),
     [
         ("/", None, "/long-example/income_statement/"),
         ("/long-example/", None, "/long-example/income_statement/"),
@@ -116,7 +119,7 @@ def test_default_path_redirection(
     url: str,
     option: str | None,
     expect: str,
-    monkeypatch: MonkeyPatch,
+    monkeypatch: pytest.MonkeyPatch,
 ) -> None:
     """Test that default-page option redirects as expected."""
     with app.test_request_context("/long-example/"):
@@ -125,17 +128,18 @@ def test_default_path_redirection(
             monkeypatch.setattr(g.ledger.fava_options, "default_page", option)
         result = test_client.get(url)
         get_url = result.headers.get("Location", "")
-        expect_url = werkzeug.urls.url_join("http://localhost/", expect)
+        # pre Werkzeug 2.1:
+        expect_url = urljoin("http://localhost/", expect)
         assert result.status_code == 302
-        assert get_url == expect_url
+        assert get_url in {expect, expect_url}
 
 
 @pytest.mark.parametrize(
-    "referer,jump_link,expect",
+    ("referer", "jump_link", "expect"),
     [
         ("/?foo=bar", "/jump?foo=baz", "/?foo=baz"),
-        ("/?foo=bar", "/jump?baz=qux", "/?baz=qux&foo=bar"),
-        ("/", "/jump?foo=bar&baz=qux", "/?baz=qux&foo=bar"),
+        ("/?foo=bar", "/jump?baz=qux", "/?foo=bar&baz=qux"),
+        ("/", "/jump?foo=bar&baz=qux", "/?foo=bar&baz=qux"),
         ("/", "/jump?baz=qux", "/?baz=qux"),
         ("/?foo=bar", "/jump?foo=", "/"),
         ("/?foo=bar", "/jump?foo=&foo=", "/?foo=&foo="),
@@ -156,17 +160,18 @@ def test_jump_handler(
     result = test_client.get(jump_link, headers=[("Referer", referer)])
     with app.test_request_context():
         get_url = result.headers.get("Location", "")
-        expect_url = werkzeug.urls.url_join("http://localhost/", expect)
+        # pre Werkzeug 2.1:
+        expect_url = urljoin("http://localhost/", expect)
         assert result.status_code == 302
-        assert get_url == expect_url
+        assert get_url in {expect, expect_url}
 
 
-def test_help_ages(test_client: FlaskClient) -> None:
+def test_help_pages(test_client: FlaskClient) -> None:
     """Help pages."""
     result = test_client.get("/long-example/help/")
     assert result.status_code == 200
-    assert f"Fava <code>{fava_version}</code>" in result.get_data(True)
-    assert f"<code>{beancount_version}</code>" in result.get_data(True)
+    assert f"Fava <code>{fava_version}</code>" in result.get_data(as_text=True)
+    assert f"<code>{beancount_version}</code>" in result.get_data(as_text=True)
     result = test_client.get("/long-example/help/filters")
     assert result.status_code == 200
     result = test_client.get("/long-example/help/asdfasdf")
@@ -176,32 +181,53 @@ def test_help_ages(test_client: FlaskClient) -> None:
 def test_query_download(test_client: FlaskClient) -> None:
     """Download query as csv."""
     result = test_client.get(
-        "/long-example/download-query/query_result.csv?query_string=balances"
+        "/long-example/download-query/query_result.csv?query_string=balances",
     )
     assert result.status_code == 200
 
 
-def test_incognito(
-    app: Flask, test_client: FlaskClient, monkeypatch: MonkeyPatch
-) -> None:
+def test_incognito(test_data_dir: Path) -> None:
     """Numbers get obfuscated in incognito mode."""
-    monkeypatch.setitem(app.config, "INCOGNITO", True)
-    result = test_client.get("/long-example/balance_sheet/")
+    app = create_app([test_data_dir / "example.beancount"], incognito=True)
+    test_client = app.test_client()
+    result = test_client.get("/example/balance_sheet/")
     assert result.status_code == 200
-    assert "XXX" in result.get_data(True)
+    assert "XXX" in result.get_data(as_text=True)
+
+    result = test_client.get("/example/api/commodities")
+    assert result.status_code == 200
+    assert "XXX" not in result.get_data(as_text=True)
+
+
+def test_read_only_mode(test_data_dir: Path) -> None:
+    """Non GET requests returns 401 in read-only mode"""
+    app = create_app([test_data_dir / "example.beancount"], read_only=True)
+    test_client = app.test_client()
+
+    assert test_client.get("/").status_code == 302
+
+    for method in [
+        test_client.delete,
+        test_client.patch,
+        test_client.post,
+        test_client.put,
+    ]:
+        result = method("/any/path/")
+        assert result.status_code == 401
 
 
 def test_download_journal(
-    app: Flask, test_client: FlaskClient, snapshot: SnapshotFunc
+    test_client: FlaskClient,
+    snapshot: SnapshotFunc,
 ) -> None:
     """The currently filtered journal can be downloaded."""
-    with app.test_request_context("/long-example/"):
-        app.preprocess_request()
-        url = url_for("download_journal", time="2016-05-07")
-    result = test_client.get(url)
-    snapshot(result.get_data(True))
+    result = test_client.get(
+        "/long-example/download-journal/",
+        query_string={"time": "2016-05-07"},
+    )
+    snapshot(result.get_data(as_text=True))
     assert result.headers["Content-Disposition"].startswith(
-        'attachment; filename="journal_'
+        'attachment; filename="journal_',
     )
     assert result.headers["Content-Type"] == "application/octet-stream"
 
@@ -209,22 +235,24 @@ def test_download_journal(
 def test_static_url(app: Flask) -> None:
     """Static URLs have the mtime appended."""
     with app.test_request_context():
-        app.preprocess_request()
         url = static_url("app.js")
-    assert url.startswith("/static/app.js?mtime=")
+        assert url.startswith("/static/app.js?mtime=")
+        url = static_url("nonexistent.js")
+        assert url == "/static/nonexistent.js?mtime=0"
 
 
-def test_load_extension_reports(app: Flask, test_client: FlaskClient) -> None:
+def test_load_extension_reports(test_client: FlaskClient) -> None:
     """Extension can register reports."""
-    with app.test_request_context("/extension-report-beancount-file/"):
-        app.preprocess_request()
-        assert g.ledger.extensions.reports == [
-            ("PortfolioList", "Portfolio List")
-        ]
 
-        url = url_for("extension_report", report_name="PortfolioList")
-        result = test_client.get(url)
-        assert result.status_code == 200
-        url = url_for("extension_report", report_name="MissingExtension")
-        result = test_client.get(url)
-        assert result.status_code == 404
+    url = "/extension-report/extension/PortfolioList/"
+    result = test_client.get(url)
+    assert result.status_code == 200
+    url = "/extension-report/extension_js_module/PortfolioList.js"
+    result = test_client.get(url)
+    assert result.status_code == 200
+    url = "/extension-report/extension_js_module/Missing.js"
+    result = test_client.get(url)
+    assert result.status_code == 404
+    url = "/extension-report/extension/MissingExtension/"
+    result = test_client.get(url)
+    assert result.status_code == 404
