@@ -1,9 +1,16 @@
-# pylint: disable=missing-docstring
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from fava.core.attributes import get_active_years
 from fava.util.date import FiscalYearEnd
 
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.beans.abc import Directive
+    from fava.core import FavaLedger
 
-def test_get_active_years(load_doc) -> None:
+
+def test_get_active_years(load_doc_entries: list[Directive]) -> None:
     """
     2010-11-12 * "test"
         Assets:T   4.00 USD
@@ -15,26 +22,24 @@ def test_get_active_years(load_doc) -> None:
         Assets:T   4.00 USD
         Expenses:T
     """
-
-    entries, _, __ = load_doc
-    assert get_active_years(entries, FiscalYearEnd(12, 31)) == [
+    assert get_active_years(load_doc_entries, FiscalYearEnd(12, 31)) == [
         "2012",
         "2011",
         "2010",
     ]
-    assert get_active_years(entries, FiscalYearEnd(12, 1)) == [
+    assert get_active_years(load_doc_entries, FiscalYearEnd(12, 1)) == [
         "FY2013",
         "FY2011",
         "FY2010",
     ]
-    assert get_active_years(entries, FiscalYearEnd(11, 1)) == [
+    assert get_active_years(load_doc_entries, FiscalYearEnd(11, 1)) == [
         "FY2013",
         "FY2012",
         "FY2011",
     ]
 
 
-def test_payee_accounts(example_ledger) -> None:
+def test_payee_accounts(example_ledger: FavaLedger) -> None:
     attr = example_ledger.attributes
     assert attr.payee_accounts("NOTAPAYEE") == attr.accounts
 
@@ -43,8 +48,10 @@ def test_payee_accounts(example_ledger) -> None:
     assert len(verizon) == len(attr.accounts)
 
 
-def test_payee_transaction(example_ledger) -> None:
+def test_payee_transaction(example_ledger: FavaLedger) -> None:
     attr = example_ledger.attributes
     assert attr.payee_transaction("NOTAPAYEE") is None
 
-    assert str(attr.payee_transaction("BayBook").date) == "2016-05-05"
+    txn = attr.payee_transaction("BayBook")
+    assert txn
+    assert str(txn.date) == "2016-05-05"
