@@ -1,25 +1,30 @@
-# pylint: disable=missing-docstring
 from __future__ import annotations
 
 import re
 from datetime import date
 from pathlib import Path
 from textwrap import dedent
+from typing import TYPE_CHECKING
 
 import pytest
-from beancount.core.amount import A
-from beancount.core.data import Posting
-from beancount.core.data import Transaction
 
-from .conftest import SnapshotFunc
-from fava.core import FavaLedger
+from fava.beans import create
+from fava.beans.funcs import get_position
+from fava.beans.helpers import replace
 from fava.core.fava_options import InsertEntryOption
+from fava.core.file import delete_entry_slice
+from fava.core.file import ExternallyChangedError
 from fava.core.file import find_entry_lines
 from fava.core.file import get_entry_slice
 from fava.core.file import insert_entry
 from fava.core.file import insert_metadata_in_file
 from fava.core.file import save_entry_slice
-from fava.helpers import FavaAPIException
+
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.beans.abc import Transaction
+    from fava.core import FavaLedger
+
+    from .conftest import SnapshotFunc
 
 
 def _get_entry(ledger: FavaLedger, payee: str, date_: str) -> Transaction:
@@ -48,12 +53,12 @@ def test_save_entry_slice(example_ledger: FavaLedger) -> None:
     entry_source, sha256sum = get_entry_slice(entry)
     new_source = """2016-05-03 * "Chichipotle" "Eating out with Joe"
   Expenses:Food:Restaurant                          21.70 USD"""
-    filename = Path(entry.meta["filename"])
+    filename = Path(get_position(entry)[0])
     contents = filename.read_text("utf-8")
 
-    with pytest.raises(FavaAPIException):
+    with pytest.raises(ExternallyChangedError):
         save_entry_slice(entry, new_source, "wrong hash")
-        assert filename.read_text("utf-8") == contents
+    assert filename.read_text("utf-8") == contents
 
     new_sha256sum = save_entry_slice(entry, new_source, sha256sum)
     assert filename.read_text("utf-8") != contents
@@ -61,47 +66,65 @@ def test_save_entry_slice(example_ledger: FavaLedger) -> None:
     assert filename.read_text("utf-8") == contents
 
 
+def test_delete_entry_slice(example_ledger: FavaLedger) -> None:
+    entry = _get_entry(example_ledger, "Chichipotle", "2016-05-03")
+
+    _entry_source, sha256sum = get_entry_slice(entry)
+    filename, lineno = get_position(entry)
+    path = Path(filename)
+    contents = path.read_text("utf-8")
+
+    with pytest.raises(ExternallyChangedError):
+        delete_entry_slice(entry, "wrong hash")
+    assert path.read_text("utf-8") == contents
+
+    delete_entry_slice(entry, sha256sum)
+    assert path.read_text("utf-8") != contents
+
+    insert_option = InsertEntryOption(
+        date(1, 1, 1),
+        re.compile(".*"),
+        filename,
+        lineno,
+    )
+    insert_entry(entry, filename, [insert_option], 59, 2)
+    assert path.read_text("utf-8") == contents
+
+
 def test_insert_metadata_in_file(tmp_path: Path) -> None:
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
     samplefile = tmp_path / "example.beancount"
     samplefile.write_text(file_content)
 
     # Insert some metadata lines.
-    insert_metadata_in_file(str(samplefile), 1, 4, "metadata", "test1")
-    insert_metadata_in_file(str(samplefile), 1, 4, "metadata", "test2")
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    insert_metadata_in_file(samplefile, 1, 4, "metadata", "test1")
+    insert_metadata_in_file(samplefile, 1, 4, "metadata", "test2")
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             metadata: "test2"
             metadata: "test1"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
 
     # Check that inserting also works if the next line is empty.
-    insert_metadata_in_file(str(samplefile), 5, 4, "metadata", "test1")
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    insert_metadata_in_file(samplefile, 5, 4, "metadata", "test1")
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             metadata: "test2"
             metadata: "test1"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
             metadata: "test1"
-        """
-    )
+        """)
 
 
 def test_find_entry_lines() -> None:
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
@@ -112,8 +135,7 @@ def test_find_entry_lines() -> None:
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
     lines = file_content.split("\n")
     entry_lines = [
         '2016-02-26 * "Uncle Boons" "Eating out alone"',
@@ -128,29 +150,23 @@ def test_find_entry_lines() -> None:
 
 
 def test_insert_entry_transaction(tmp_path: Path) -> None:
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
     samplefile = tmp_path / "example.beancount"
     samplefile.write_text(file_content)
 
     postings = [
-        Posting(
+        create.posting(
             "Liabilities:US:Chase:Slate",
-            A("-10.00 USD"),
-            None,
-            None,
-            None,
-            None,
+            "-10.00 USD",
         ),
-        Posting("Expenses:Food", A("10.00 USD"), None, None, None, None),
+        create.posting("Expenses:Food", "10.00 USD"),
     ]
 
-    transaction = Transaction(
+    transaction = create.transaction(
         {},
         date(2016, 1, 1),
         "*",
@@ -163,8 +179,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
 
     # Test insertion without "insert-entry" options.
     insert_entry(transaction, str(samplefile), [], 61, 4)
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
@@ -172,8 +187,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
-        """
-    )
+        """)
 
     # Verify that InsertEntryOptions with dates greater or equal than the
     # transaction dates are ignored.
@@ -198,7 +212,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
         ),
     ]
     new_options = insert_entry(
-        transaction._replace(narration="narr1"),
+        replace(transaction, narration="narr1"),
         str(samplefile),
         options,
         61,
@@ -207,8 +221,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
     assert new_options[0].lineno == 5
     assert new_options[1].lineno == 5
     assert new_options[2].lineno == 10
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-01-01 * "new payee" "narr1"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
@@ -220,8 +233,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
-        """
-    )
+        """)
 
     # Verify that previous postings are matched against InsertEntryOptions when
     # the last posting doesn't match.
@@ -239,12 +251,17 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
             1,
         ),
     ]
-    transaction = transaction._replace(narration="narr2")
-    new_options = insert_entry(transaction, str(samplefile), options, 61, 4)
+    new_transaction = replace(transaction, narration="narr2")
+    new_options = insert_entry(
+        new_transaction,
+        str(samplefile),
+        options,
+        61,
+        4,
+    )
     assert new_options[0].lineno == 9
     assert new_options[1].lineno == 1
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-01-01 * "new payee" "narr1"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
@@ -260,8 +277,7 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
-        """
-    )
+        """)
 
     # Verify that preference is given to InsertEntryOptions with later dates in
     # case several of them match a posting.
@@ -279,10 +295,9 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
             1,
         ),
     ]
-    transaction = transaction._replace(narration="narr3")
-    insert_entry(transaction, str(samplefile), options, 61, 4)
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    new_transaction = replace(transaction, narration="narr3")
+    insert_entry(new_transaction, str(samplefile), options, 61, 4)
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-01-01 * "new payee" "narr3"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
@@ -302,34 +317,27 @@ def test_insert_entry_transaction(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
             Liabilities:US:Chase:Slate                       -10.00 USD
             Expenses:Food                                     10.00 USD
-        """
-    )
+        """)
 
 
 def test_insert_entry_align(tmp_path: Path) -> None:
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
     samplefile = tmp_path / "example.beancount"
     samplefile.write_text(file_content)
 
     postings = [
-        Posting(
+        create.posting(
             "Liabilities:US:Chase:Slate",
-            A("-10.00 USD"),
-            None,
-            None,
-            None,
-            None,
+            "-10.00 USD",
         ),
-        Posting("Expenses:Food", A("10.00 USD"), None, None, None, None),
+        create.posting("Expenses:Food", "10.00 USD"),
     ]
 
-    transaction = Transaction(
+    transaction = create.transaction(
         {},
         date(2016, 1, 1),
         "*",
@@ -341,8 +349,7 @@ def test_insert_entry_align(tmp_path: Path) -> None:
     )
 
     insert_entry(transaction, str(samplefile), [], 50, 4)
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
@@ -350,34 +357,27 @@ def test_insert_entry_align(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
             Liabilities:US:Chase:Slate            -10.00 USD
             Expenses:Food                          10.00 USD
-        """
-    )
+        """)
 
 
 def test_insert_entry_indent(tmp_path: Path) -> None:
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
-        """
-    )
+        """)
     samplefile = tmp_path / "example.beancount"
     samplefile.write_text(file_content)
 
     postings = [
-        Posting(
+        create.posting(
             "Liabilities:US:Chase:Slate",
-            A("-10.00 USD"),
-            None,
-            None,
-            None,
-            None,
+            "-10.00 USD",
         ),
-        Posting("Expenses:Food", A("10.00 USD"), None, None, None, None),
+        create.posting("Expenses:Food", "10.00 USD"),
     ]
 
-    transaction = Transaction(
+    transaction = create.transaction(
         {},
         date(2016, 1, 1),
         "*",
@@ -390,8 +390,7 @@ def test_insert_entry_indent(tmp_path: Path) -> None:
 
     # Test insertion with 2-space indent.
     insert_entry(transaction, str(samplefile), [], 61, 2)
-    assert samplefile.read_text("utf-8") == dedent(
-        """\
+    assert samplefile.read_text("utf-8") == dedent("""\
         2016-02-26 * "Uncle Boons" "Eating out alone"
             Liabilities:US:Chase:Slate                       -24.84 USD
             Expenses:Food:Restaurant                          24.84 USD
@@ -399,19 +398,19 @@ def test_insert_entry_indent(tmp_path: Path) -> None:
         2016-01-01 * "new payee" "narr"
           Liabilities:US:Chase:Slate                         -10.00 USD
           Expenses:Food                                       10.00 USD
-        """
-    )
+        """)
 
 
 def test_render_entries(
-    example_ledger: FavaLedger, snapshot: SnapshotFunc
+    example_ledger: FavaLedger,
+    snapshot: SnapshotFunc,
 ) -> None:
     entry1 = _get_entry(example_ledger, "Uncle Boons", "2016-04-09")
     entry2 = _get_entry(example_ledger, "BANK FEES", "2016-05-04")
     postings = [
-        Posting("Expenses:Food", A("10.00 USD"), None, None, None, None),
+        create.posting("Expenses:Food", "10.00 USD"),
     ]
-    transaction = Transaction(
+    transaction = create.transaction(
         {},
         date(2016, 1, 1),
         "*",
@@ -424,8 +423,7 @@ def test_render_entries(
     entries = example_ledger.file.render_entries([entry1, entry2, transaction])
     snapshot("\n".join(entries))
 
-    file_content = dedent(
-        """\
+    file_content = dedent("""\
         2016-04-09 * "Uncle Boons" "" #trip-new-york-2016
           Liabilities:US:Chase:Slate                       -52.22 USD
           Expenses:Food:Restaurant                          52.22 USD
@@ -433,9 +431,8 @@ def test_render_entries(
         2016-05-04 * "BANK FEES" "Monthly bank fee"
           Assets:US:BofA:Checking                           -4.00 USD
           Expenses:Financial:Fees                            4.00 USD
-        """
-    )
+        """)
 
     assert file_content == "\n".join(
-        example_ledger.file.render_entries([entry1, entry2])
+        example_ledger.file.render_entries([entry1, entry2]),
     )
