@@ -1,22 +1,25 @@
-# pylint: disable=missing-docstring
+from __future__ import annotations
+
 import datetime
+from typing import TYPE_CHECKING
 
 import pytest
-from beancount.core import account
-from beancount.core.data import create_simple_posting
-from beancount.core.data import Transaction
+from beancount.core.account import has_component
 
+from fava.beans import create
+from fava.beans.account import get_entry_accounts
 from fava.core.filters import AccountFilter
 from fava.core.filters import AdvancedFilter
-from fava.core.filters import FilterException
+from fava.core.filters import FilterError
 from fava.core.filters import FilterSyntaxLexer
 from fava.core.filters import Match
 from fava.core.filters import TimeFilter
 
-LEX = FilterSyntaxLexer().lex
+if TYPE_CHECKING:  # pragma: no cover
+    from fava.core import FavaLedger
 
 
-def test_match():
+def test_match() -> None:
     assert Match("asdf")("asdf")
     assert Match("asdf")("asdfasdf")
     assert Match("asdf")("aasdfasdf")
@@ -26,35 +29,38 @@ def test_match():
     assert Match("(((")("(((")
 
 
-def test_lexer_basic():
+def test_lexer_basic() -> None:
+    lex = FilterSyntaxLexer().lex
     data = "#some_tag ^some_link -^some_link"
-    assert [(tok.type, tok.value) for tok in LEX(data)] == [
+    assert [(tok.type, tok.value) for tok in lex(data)] == [
         ("TAG", "some_tag"),
         ("LINK", "some_link"),
         ("-", "-"),
         ("LINK", "some_link"),
     ]
     data = "'string' string \"string\""
-    assert [(tok.type, tok.value) for tok in LEX(data)] == [
+    assert [(tok.type, tok.value) for tok in lex(data)] == [
         ("STRING", "string"),
         ("STRING", "string"),
         ("STRING", "string"),
     ]
-    with pytest.raises(FilterException):
-        list(LEX("|"))
+    with pytest.raises(FilterError):
+        list(lex("|"))
 
 
-def test_lexer_literals_in_string():
+def test_lexer_literals_in_string() -> None:
+    lex = FilterSyntaxLexer().lex
     data = "string-2-2 string"
-    assert [(tok.type, tok.value) for tok in LEX(data)] == [
+    assert [(tok.type, tok.value) for tok in lex(data)] == [
         ("STRING", "string-2-2"),
         ("STRING", "string"),
     ]
 
 
-def test_lexer_key():
+def test_lexer_key() -> None:
+    lex = FilterSyntaxLexer().lex
     data = 'payee:asdfasdf ^some_link somekey:"testtest" '
-    assert [(tok.type, tok.value) for tok in LEX(data)] == [
+    assert [(tok.type, tok.value) for tok in lex(data)] == [
         ("KEY", "payee"),
         ("STRING", "asdfasdf"),
         ("LINK", "some_link"),
@@ -63,9 +69,10 @@ def test_lexer_key():
     ]
 
 
-def test_lexer_parentheses():
+def test_lexer_parentheses() -> None:
+    lex = FilterSyntaxLexer().lex
     data = "(payee:asdfasdf ^some_link) (somekey:'testtest')"
-    assert [(tok.type, tok.value) for tok in LEX(data)] == [
+    assert [(tok.type, tok.value) for tok in lex(data)] == [
         ("(", "("),
         ("KEY", "payee"),
         ("STRING", "asdfasdf"),
@@ -78,27 +85,16 @@ def test_lexer_parentheses():
     ]
 
 
-FILTER = AdvancedFilter({}, {})
+def test_filterexception() -> None:
+    with pytest.raises(FilterError, match='Illegal character """ in filter'):
+        AdvancedFilter('who:"fff')
 
-
-def test_filterexception():
-    with pytest.raises(FilterException) as exception:
-        raise FilterException("type", "error")
-    exception = exception.value
-    assert str(exception) == "error"
-    assert str(exception) == exception.message
-
-    with pytest.raises(FilterException):
-        FILTER.set('who:"fff')
-        assert str(exception) == 'Illegal character """ in filter: who:"fff'
-
-    with pytest.raises(FilterException):
-        FILTER.set('any(who:"Martin"')
-        assert str(exception) == 'Failed to parse filter: any(who:"Martin"'
+    with pytest.raises(FilterError, match="Failed to parse filter"):
+        AdvancedFilter('any(who:"Martin"')
 
 
 @pytest.mark.parametrize(
-    "string,number",
+    ("string", "number"),
     [
         ('any(account:"Assets:US:ETrade")', 48),
         ('all(-account:"Assets:US:ETrade")', 1826 - 48),
@@ -125,76 +121,74 @@ def test_filterexception():
         ('any(overage:"GB$")', 1),
     ],
 )
-def test_advanced_filter(example_ledger, string, number):
-    FILTER.set(string)
-    filtered_entries = FILTER.apply(example_ledger.all_entries)
+def test_advanced_filter(
+    example_ledger: FavaLedger,
+    string: str,
+    number: int,
+) -> None:
+    filter_ = AdvancedFilter(string)
+    filtered_entries = filter_.apply(example_ledger.all_entries)
     assert len(filtered_entries) == number
 
 
-def test_null_meta_posting():
-    FILTER.set('any(some_meta:"1")')
+def test_null_meta_posting() -> None:
+    filter_ = AdvancedFilter('any(some_meta:"1")')
 
-    txn = Transaction(
+    txn = create.transaction(
         {},
         datetime.date(2017, 12, 12),
         "*",
         "",
         "",
-        None,
-        None,
-        [],
+        frozenset(),
+        frozenset(),
+        [create.posting("Assets:ETrade:Cash", "100 USD")],
     )
-    # This will create a posting with meta set to `None`.
-    create_simple_posting(txn, "Assets:ETrade:Cash", "100", "USD")
     assert txn.postings[0].meta is None
-    assert len(FILTER.apply([txn])) == 0
+    assert len(filter_.apply([txn])) == 0
 
 
-def test_account_filter(example_ledger):
-    account_filter = AccountFilter(
-        example_ledger.options, example_ledger.fava_options
-    )
+def test_account_filter(example_ledger: FavaLedger) -> None:
+    account_filter = AccountFilter("Assets")
 
-    account_filter.set("Assets")
     filtered_entries = account_filter.apply(example_ledger.all_entries)
     assert len(filtered_entries) == 541
-    assert all(
-        map(
-            lambda x: hasattr(x, "account")
-            and account.has_component(x.account, "Assets")
-            or any(
-                map(
-                    lambda p: account.has_component(p.account, "Assets"),
-                    x.postings,
-                )
-            ),
-            filtered_entries,
-        )
-    )
 
-    account_filter.set(".*US:State")
+    for entry in filtered_entries:
+        assert any(
+            has_component(a, "Assets") for a in get_entry_accounts(entry)
+        )
+
+    account_filter = AccountFilter(".*US:State")
     filtered_entries = account_filter.apply(example_ledger.all_entries)
     assert len(filtered_entries) == 67
 
 
-def test_time_filter(example_ledger):
+def test_time_filter(example_ledger: FavaLedger) -> None:
     time_filter = TimeFilter(
-        example_ledger.options, example_ledger.fava_options
+        example_ledger.options,
+        example_ledger.fava_options,
+        "2017",
     )
 
-    time_filter.set("2017")
-    assert time_filter.begin_date == datetime.date(2017, 1, 1)
-    assert time_filter.end_date == datetime.date(2018, 1, 1)
+    date_range = time_filter.date_range
+    assert date_range
+    assert date_range.begin == datetime.date(2017, 1, 1)
+    assert date_range.end == datetime.date(2018, 1, 1)
     filtered_entries = time_filter.apply(example_ledger.all_entries)
     assert len(filtered_entries) == 83
 
-    time_filter.set("1000")
+    time_filter = TimeFilter(
+        example_ledger.options,
+        example_ledger.fava_options,
+        "1000",
+    )
     filtered_entries = time_filter.apply(example_ledger.all_entries)
     assert not filtered_entries
 
-    time_filter.set(None)
-    filtered_entries = time_filter.apply(example_ledger.all_entries)
-    assert len(filtered_entries) == len(example_ledger.all_entries)
-
-    with pytest.raises(FilterException):
-        time_filter.set("no_date")
+    with pytest.raises(FilterError):
+        TimeFilter(
+            example_ledger.options,
+            example_ledger.fava_options,
+            "no_date",
+        )
