@@ -1,30 +1,32 @@
-# pylint: disable=missing-docstring
+from __future__ import annotations
+
+from typing import Iterable
+from typing import TYPE_CHECKING
+
 from werkzeug.test import Client
 from werkzeug.wrappers import Response
 
-from .conftest import data_file
 from fava.util import listify
 from fava.util import next_key
-from fava.util import pairwise
 from fava.util import send_file_inline
 from fava.util import simple_wsgi
 from fava.util import slugify
 
+if TYPE_CHECKING:  # pragma: no cover
+    from pathlib import Path
 
-def test_listify():
+    from flask import Flask
+
+
+def test_listify() -> None:
     @listify
-    def fun():
+    def fun() -> Iterable[int]:
         yield from [1, 2, 3]
 
     assert fun() == [1, 2, 3]
 
 
-def test_pairwise():
-    assert list(pairwise([1, 2, 3])) == [(1, 2), (2, 3)]
-    assert list(pairwise([])) == []
-
-
-def test_simple_wsgi():
+def test_simple_wsgi() -> None:
     client = Client(simple_wsgi, Response)
     resp = client.get("/any_path")
     assert resp.status_code == 200
@@ -41,27 +43,26 @@ def test_next_key() -> None:
     )
 
 
-def test_slugify():
+def test_slugify() -> None:
     assert slugify("Example Beancount File") == "example-beancount-file"
     assert slugify("    Example Beancount File  ") == "example-beancount-file"
     assert slugify("test") == "test"
     assert slugify("烫烫烫") == "烫烫烫"
     assert slugify("nonun烫icode 烫烫") == "nonun烫icode-烫烫"
-    assert slugify("%✓") == ""
+    assert not slugify("%✓")
     assert slugify("söße") == "söße"
     assert slugify("ASDF") == "asdf"
     assert slugify("ASDF test test") == "asdf-test-test"
 
 
-def test_send_file_inline(app):
+def test_send_file_inline(app: Flask, test_data_dir: Path) -> None:
     with app.test_request_context():
-        resp = send_file_inline(data_file("example-balances.csv"))
+        resp = send_file_inline(str(test_data_dir / "example-balances.csv"))
         assert (
             resp.headers["Content-Disposition"]
             == "inline; filename*=UTF-8''example-balances.csv"
         )
-        resp = send_file_inline(data_file("example-utf8-🦁.txt"))
-        # pylint: disable=line-too-long
+        resp = send_file_inline(str(test_data_dir / "example-utf8-🦁.txt"))
         assert (
             resp.headers["Content-Disposition"]
             == "inline; filename*=UTF-8''example-utf8-%F0%9F%A6%81.txt"
