@@ -1,4 +1,3 @@
-# pylint: disable=missing-docstring
 from __future__ import annotations
 
 from datetime import date
@@ -10,6 +9,7 @@ import pytest
 from fava.util.date import FiscalYearEnd
 from fava.util.date import get_fiscal_period
 from fava.util.date import get_next_interval
+from fava.util.date import get_prev_interval
 from fava.util.date import Interval
 from fava.util.date import interval_ends
 from fava.util.date import month_offset
@@ -32,7 +32,30 @@ def _to_date(string: str) -> date:
 
 
 @pytest.mark.parametrize(
-    "input_date_string,interval,expect",
+    ("input_date_string", "interval", "expect", "expect_filter"),
+    [
+        ("2016-01-01", Interval.DAY, "2016-01-01", "2016-01-01"),
+        ("2016-01-04", Interval.WEEK, "2016W01", "2016-W01"),
+        ("2016-01-04", Interval.MONTH, "Jan 2016", "2016-01"),
+        ("2016-01-04", Interval.QUARTER, "2016Q1", "2016-Q1"),
+        ("2016-01-04", Interval.YEAR, "2016", "2016"),
+    ],
+)
+def test_interval_format(
+    input_date_string: str,
+    interval: Interval,
+    expect: str,
+    expect_filter: str,
+) -> None:
+    assert interval.format_date(_to_date(input_date_string)) == expect
+    assert (
+        interval.format_date_filter(_to_date(input_date_string))
+        == expect_filter
+    )
+
+
+@pytest.mark.parametrize(
+    ("input_date_string", "interval", "expect"),
     [
         ("2016-01-01", Interval.DAY, "2016-01-02"),
         ("2016-01-01", Interval.WEEK, "2016-01-04"),
@@ -49,28 +72,51 @@ def _to_date(string: str) -> date:
     ],
 )
 def test_get_next_interval(
-    input_date_string: str, interval: Interval, expect: str
+    input_date_string: str,
+    interval: Interval,
+    expect: str,
 ) -> None:
     get = get_next_interval(_to_date(input_date_string), interval)
     assert get == _to_date(expect)
 
 
-def test_get_next_intervalfail2() -> None:
-    with pytest.raises(NotImplementedError):
-        get_next_interval(date(2016, 4, 18), "decade")  # type: ignore
+@pytest.mark.parametrize(
+    ("input_date_string", "interval", "expect"),
+    [
+        ("2016-01-01", Interval.DAY, "2016-01-01"),
+        ("2016-01-01", Interval.WEEK, "2015-12-28"),
+        ("2016-01-01", Interval.MONTH, "2016-01-01"),
+        ("2016-01-01", Interval.QUARTER, "2016-01-01"),
+        ("2016-01-01", Interval.YEAR, "2016-01-01"),
+        ("2016-12-31", Interval.DAY, "2016-12-31"),
+        ("2016-12-31", Interval.WEEK, "2016-12-26"),
+        ("2016-12-31", Interval.MONTH, "2016-12-01"),
+        ("2016-12-31", Interval.QUARTER, "2016-10-01"),
+        ("2016-12-31", Interval.YEAR, "2016-01-01"),
+        ("9999-12-31", Interval.QUARTER, "9999-10-01"),
+        ("9999-12-31", Interval.YEAR, "9999-01-01"),
+    ],
+)
+def test_get_prev_interval(
+    input_date_string: str,
+    interval: Interval,
+    expect: str,
+) -> None:
+    get = get_prev_interval(_to_date(input_date_string), interval)
+    assert get == _to_date(expect)
 
 
 def test_interval_tuples() -> None:
     assert list(
-        interval_ends(date(2014, 3, 5), date(2014, 5, 5), Interval.MONTH)
+        interval_ends(date(2014, 3, 5), date(2014, 5, 5), Interval.MONTH),
     ) == [
-        date(2014, 3, 5),
+        date(2014, 3, 1),
         date(2014, 4, 1),
         date(2014, 5, 1),
-        date(2014, 5, 5),
+        date(2014, 6, 1),
     ]
     assert list(
-        interval_ends(date(2014, 1, 1), date(2014, 5, 1), Interval.MONTH)
+        interval_ends(date(2014, 1, 1), date(2014, 5, 1), Interval.MONTH),
     ) == [
         date(2014, 1, 1),
         date(2014, 2, 1),
@@ -79,15 +125,15 @@ def test_interval_tuples() -> None:
         date(2014, 5, 1),
     ]
     assert list(
-        interval_ends(date(2014, 3, 5), date(2014, 5, 5), Interval.YEAR)
-    ) == [date(2014, 3, 5), date(2014, 5, 5)]
+        interval_ends(date(2014, 3, 5), date(2014, 5, 5), Interval.YEAR),
+    ) == [date(2014, 1, 1), date(2015, 1, 1)]
     assert list(
-        interval_ends(date(2014, 1, 1), date(2015, 1, 1), Interval.YEAR)
+        interval_ends(date(2014, 1, 1), date(2015, 1, 1), Interval.YEAR),
     ) == [date(2014, 1, 1), date(2015, 1, 1)]
 
 
 @pytest.mark.parametrize(
-    "string,output",
+    ("string", "output"),
     [
         ("year", "2016"),
         ("(year-1)", "2015"),
@@ -120,7 +166,7 @@ def test_substitute(string: str, output: str) -> None:
 
 
 @pytest.mark.parametrize(
-    "fye_str,test_date,string,output",
+    ("fye_str", "test_date", "string", "output"),
     [
         ("06-30", "2018-02-02", "fiscal_year", "FY2018"),
         ("06-30", "2018-08-02", "fiscal_year", "FY2019"),
@@ -138,21 +184,27 @@ def test_substitute(string: str, output: str) -> None:
     ],
 )
 def test_fiscal_substitute(
-    fye_str: str, test_date: str, string: str, output: str | None
+    fye_str: str,
+    test_date: str,
+    string: str,
+    output: str | None,
 ) -> None:
     fye = parse_fye_string(fye_str)
     with mock.patch("fava.util.date.datetime.date") as mock_date:
         mock_date.today.return_value = _to_date(test_date)
         mock_date.side_effect = date
         if output is None:
-            with pytest.raises(ValueError):
+            with pytest.raises(
+                ValueError,
+                match="Cannot use fiscal_quarter if fiscal year",
+            ):
                 substitute(string, fye)
         else:
             assert substitute(string, fye) == output
 
 
 @pytest.mark.parametrize(
-    "expect_start,expect_end,text",
+    ("expect_start", "expect_end", "text"),
     [
         ("2000-01-01", "2001-01-01", "   2000   "),
         ("2010-10-01", "2010-11-01", "2010-10"),
@@ -181,7 +233,7 @@ def test_parse_date_empty() -> None:
 
 
 @pytest.mark.parametrize(
-    "expect_start,expect_end,text",
+    ("expect_start", "expect_end", "text"),
     [
         ("2014-01-01", "2016-06-27", "year-2-day+2"),
         ("2016-01-01", "2016-06-25", "year-day"),
@@ -192,7 +244,9 @@ def test_parse_date_empty() -> None:
     ],
 )
 def test_parse_date_relative(
-    expect_start: str, expect_end: str, text: str
+    expect_start: str,
+    expect_end: str,
+    text: str,
 ) -> None:
     start, end = _to_date(expect_start), _to_date(expect_end)
     with mock.patch("fava.util.date.datetime.date") as mock_date:
@@ -202,7 +256,7 @@ def test_parse_date_relative(
 
 
 @pytest.mark.parametrize(
-    "interval,date_str,expect",
+    ("interval", "date_str", "expect"),
     [
         (Interval.DAY, "2016-05-01", 1),
         (Interval.DAY, "2016-05-31", 1),
@@ -224,18 +278,15 @@ def test_parse_date_relative(
     ],
 )
 def test_number_of_days_in_period(
-    interval: Interval, date_str: str, expect: int
+    interval: Interval,
+    date_str: str,
+    expect: int,
 ) -> None:
     assert number_of_days_in_period(interval, _to_date(date_str)) == expect
 
 
-def test_number_of_days_in_period2() -> None:
-    with pytest.raises(NotImplementedError):
-        number_of_days_in_period("test", date(2011, 2, 1))  # type: ignore
-
-
 @pytest.mark.parametrize(
-    "date_input,offset,expected",
+    ("date_input", "offset", "expected"),
     [
         ("2018-01-12", 0, "2018-01-12"),
         ("2018-01-01", -3, "2017-10-01"),
@@ -245,18 +296,20 @@ def test_number_of_days_in_period2() -> None:
     ],
 )
 def test_month_offset(
-    date_input: str, offset: int, expected: str | None
+    date_input: str,
+    offset: int,
+    expected: str | None,
 ) -> None:
     start_date = _to_date(date_input)
     if expected is None:
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="day is out of range"):
             month_offset(start_date, offset)
     else:
         assert str(month_offset(start_date, offset)) == expected
 
 
 @pytest.mark.parametrize(
-    "year,quarter,fye_str,expect_start,expect_end",
+    ("year", "quarter", "fye_str", "expect_start", "expect_end"),
     [
         # standard calendar year [FYE=12-31]
         (2018, None, "12-31", "2018-01-01", "2019-01-01"),
@@ -298,16 +351,27 @@ def test_get_fiscal_period(
 
 
 @pytest.mark.parametrize(
-    "fye,expected",
+    ("fye_str", "month", "day"),
     [
-        ("12-31", (12, 31)),
-        ("06-30", (6, 30)),
-        ("02-28", (2, 28)),
-        ("12-32", None),
-        ("asdfasdf", None),
-        ("02-29", None),
+        ("12-31", 12, 31),
+        ("06-30", 6, 30),
+        ("02-28", 2, 28),
     ],
 )
-def test_parse_fye_string(fye: str, expected: tuple[int, int] | None) -> None:
-    fye_tuple = parse_fye_string(fye)
-    assert fye_tuple == expected
+def test_parse_fye_string(fye_str: str, month: int, day: int) -> None:
+    fye = parse_fye_string(fye_str)
+    assert fye
+    assert fye.month == month
+    assert fye.day == day
+
+
+@pytest.mark.parametrize(
+    "fye_str",
+    [
+        "12-32",
+        "asdfasdf",
+        "02-29",
+    ],
+)
+def test_parse_fye_invalid_string(fye_str: str) -> None:
+    assert parse_fye_string(fye_str) is None
